-- KeY Defined : Imports
import XMonad
import XMonad.Util.EZConfig (additionalKeysP, mkNamedKeymap) -- Vim Style Key defines
import XMonad.Util.Dmenu -- Excatly what it says
import XMonad.Util.Ungrab -- For screenshots 
import XMonad.Layout.ThreeColumns -- For big scrrens
import XMonad.Hooks.EwmhDesktops
import XMonad.Layout.Spacing
import XMonad.Layout.LayoutModifier
import XMonad.Layout.Gaps



-- Default defined
import Data.Monoid
import System.Exit
import XMonad.Layout.NoBorders
import XMonad.Hooks.ManageDocks
import XMonad.Hooks.ManageHelpers
import XMonad.Util.Run
import XMonad.Util.SpawnOnce
import XMonad.Actions.Navigation2D
import XMonad.Util.NamedActions
import Data.Maybe (fromJust)
import XMonad.Layout.ShowWName
import XMonad.Layout.ThreeColumns
import qualified XMonad.StackSet as W
import qualified Data.Map        as M

-- KeY Defined : Terminal
myTerminal :: String
myTerminal = "alacritty"

-- KeY Defined : Shell interpreter
myShell :: String
myShell = "fish"

-- KeY Defined default web browser the sapce at the end is important.
myBrowser :: String
myBrowser = "qutebrowser "

-- KeY Defined default App Launcher.
myLauncher :: String
myLauncher = "dmenu_run "

-- KeY Defined Whether focus follows the mouse pointer.
myFocusFollowsMouse :: Bool
myFocusFollowsMouse = True

-- KeY Defined : Whether clicking on a window to focus also passes the click to the window
myClickJustFocuses :: Bool
myClickJustFocuses = False

-- KeY Defined : Width of the window border in pixels.
myBorderWidth   = 3

-- KeY Defined : "windows key" =  mod4Mask.
myModMask       = mod4Mask

-- myWorkspaces = [" 1 ", " 2 ", " 3 ", " 4 ", " 5 ", " 6 ", " 7 ", " 8 ", " 9 "]
myWorkspaces = [" dev ", " www ", " sys ", " doc ", " vbox ", " chat ", " mus ", " vid ", " gfx "]
myWorkspaceIndices = M.fromList $ zipWith (,) myWorkspaces [1..] -- (,) == \x y -> (x,y)

clickable ws = "<action=xdotool key super+"++show i++">"++ws++"</action>"
    where i = fromJust $ M.lookup ws myWorkspaceIndices

-- KeY Defined : Border colors for unfocused and focused windows, respectively.
myNormalBorderColor  = "#dddddd" -- White
myFocusedBorderColor = "#F000FF" -- Purple


------------------------------------------------------------------------
-- Key Defined : Key bindings. Add, modify or remove key bindings here.
--
myKeys conf@(XConfig {XMonad.modMask = modm}) = M.fromList $

    -- KeY Defined : launch default terminal with the default Shell
    [ ((modm, xK_Return), spawn $ (myTerminal ++ " -e " ++ myShell))
		
	-- KeY Defined : Volume Up
    , ((modm, xK_F3     ), spawn "amixer set Master 2%+")

	-- KeY Defined : Volume Up
    , ((modm, xK_F2     ), spawn "amixer set Master 2%-")

	-- KeY Defined : Toggle mute
    , ((modm, xK_F1     ), spawn "amixer -D pulse set Master 1+ toggle")

	-- KeY Defined : Launch (p)rogramms
    , ((modm,               xK_p     ), spawn $ (myLauncher))
    
	-- KeY Defined : Launch (i)nternet Explorer 
    , ((modm,               xK_i     ), spawn $ (myBrowser))

    -- KeY Defined : (c)lose focused window
    , ((modm,				xK_c     ), kill)

     -- KeY Defined : Rotate through the available layout algorithms
    , ((modm,               xK_space ), sendMessage NextLayout)

    --  KeY Defined : Reset the layouts on the current workspace to default
    , ((modm .|. shiftMask, xK_space ), setLayout $ XMonad.layoutHook conf)

	-- KeY Definded : Take A (S)crenn Shot
    , ((modm .|. shiftMask, xK_s     ), unGrab *> spawn "scrot -s")
    
	-- Resize viewed windows to the correct size
    , ((modm,               xK_n     ), refresh)

    -- Move focus to the next window
    , ((modm,               xK_Tab   ), windows W.focusDown)

    -- KeY Defined : Move focus to the next window
    , ((modm,               xK_j     ), windows W.focusDown)

    -- KeY Defined : Move focus to the previous window
    , ((modm,               xK_k     ), windows W.focusUp  )

    -- Move focus to the master window
    , ((modm,               xK_m     ), windows W.focusMaster  )

    -- Swap the focused window and the master window
    , ((modm .|. shiftMask, xK_Return), windows W.swapMaster)

    -- KeY Defined : Swap the focused window with the next window
    , ((modm .|. shiftMask, xK_j     ), windows W.swapDown  )

    -- KeY Defined : Swap the focused window with the previous window
    , ((modm .|. shiftMask, xK_k     ), windows W.swapUp    )

    -- KeY Defined : Shrink the master area
    , ((modm,               xK_h     ), sendMessage Shrink)

    -- KeY Defined : Expand the master area
    , ((modm,               xK_l     ), sendMessage Expand)

    -- Push window back into tiling
    , ((modm,               xK_t     ), withFocused $ windows . W.sink)

    -- Increment the number of windows in the master area
    , ((modm              , xK_comma ), sendMessage (IncMasterN 1))

    -- Deincrement the number of windows in the master area
    , ((modm              , xK_period), sendMessage (IncMasterN (-1)))

    -- KeY Defined : (Q)uit xmonad
    , ((modm .|. shiftMask, xK_q     ), io (exitWith ExitSuccess))

    -- KeY Defined : (q) Restart xmonad
    , ((modm              , xK_q     ), spawn "rm -r /home/key/.xmonad ; xmonad --recompile; xmonad --restart; pkill xmobar")

    ]
    ++

    --
    -- mod-[1..9], Switch to workspace N
    -- mod-shift-[1..9], Move client to workspace N
    --
    [((m .|. modm, k), windows $ f i)
        | (i, k) <- zip (XMonad.workspaces conf) [xK_1 .. xK_9]
        , (f, m) <- [(W.greedyView, 0), (W.shift, shiftMask)]]
    ++

    --
    -- mod-{w,e,r}, Switch to physical/Xinerama screens 1, 2, or 3
    -- mod-shift-{w,e,r}, Move client to screen 1, 2, or 3
    --
    [((m .|. modm, key), screenWorkspace sc >>= flip whenJust (windows . f))
        | (key, sc) <- zip [xK_w, xK_e, xK_r] [0..]
        , (f, m) <- [(W.view, 0), (W.shift, shiftMask)]]


------------------------------------------------------------------------
-- Mouse bindings: default actions bound to mouse events
--
myMouseBindings (XConfig {XMonad.modMask = modm}) = M.fromList $

    -- mod-button1, Set the window to floating mode and move by dragging
    [ ((modm, button1), (\w -> focus w >> mouseMoveWindow w
                                       >> windows W.shiftMaster))

    -- mod-button2, Raise the window to the top of the stack
    , ((modm, button2), (\w -> focus w >> windows W.shiftMaster))

    -- mod-button3, Set the window to floating mode and resize by dragging
    , ((modm, button3), (\w -> focus w >> mouseResizeWindow w
                                       >> windows W.shiftMaster))

    -- you may also bind events to the mouse scroll wheel (button4 and button5)
    ]

------------------------------------------------------------------------
-- Layouts:

-- You can specify and transform your layouts by modifying these values.
-- If you change layout bindings be sure to use 'mod-shift-space' after
-- restarting (with 'mod-q') to reset your layout state to the new
-- defaults, as xmonad preserves your old layout settings by default.
--
-- The available layouts.  Note that each layout is separated by |||,
-- which denotes layout choice.
--
myLayout = avoidStruts (tiled ||| Mirror tiled ||| Full)
  where
     -- default tiling algorithm partitions the screen into two panes
     tiled   = Tall nmaster delta ratio

     -- The default number of windows in the master pane
     nmaster = 1

     -- Default proportion of screen occupied by master pane
     ratio   = 1/2

     -- Percent of screen to increment by when resizing panes
     delta   = 3/100
	
------------------------------------------------------------------------
-- Window rules:

-- Execute arbitrary actions and WindowSet manipulations when managing
-- a new window. You can use this to, for example, always float a
-- particular program, or have a client always appear on a particular
-- workspace.
--
-- To find the property name associated with a program, use
-- > xprop | grep WM_CLASS
-- and click on the client you're interested in.
--
-- To match on the WM_NAME, you can use 'title' in the same way that
-- 'className' and 'resource' are used below.
--

-- KeY Defined
myManageHook = composeAll
    [ className =? "MPlayer"          --> doFloat
    , className =? "Gimp"             --> doFloat
    , className =? "Steam"            --> doFullFloat
    , className =? "factorio"            --> doFullFloat
    , className =? "TerraTechLinux64.x86_64" --> doFullFloat
    , resource  =? "desktop_window"   --> doIgnore
    , resource  =? "kdesktop"         --> doIgnore 
	, manageDocks
	]

------------------------------------------------------------------------
-- Event handling

-- * EwmhDesktops users should change this to ewmhDesktopsEventHook
--
-- Defines a custom handler function for X Events. The function should
-- return (All True) if the default handler is to be run afterwards. To
-- combine event hooks use mappend or mconcat from Data.Monoid.
--
myEventHook = mempty
------------------------------------------------------------------------
-- Status bars and logging

-- Perform an arbitrary action on each internal state change or X event.
-- See the 'XMonad.Hooks.DynamicLog' extension for examples.
--
myLogHook = mempty
------------------------------------------------------------------------
-- Startup hook

-- Perform an arbitrary action each time xmonad starts or is restarted
-- with mod-q.  Used by, e.g., XMonad.Layout.PerWorkspace to initialize
-- per-workspace layout choices.
--
-- By default, do nothing.
myStartupHook = do
 spawnOnce "compton &"
 spawnOnce "feh --bg-fill --randomize $HOME/wallpaper/* &"


------------------------------------------------------------------------
-- Now run xmonad with all the defaults we set up.

-- Run xmonad with the settings you specify. No need to modify this.
--
main :: IO () 
main = do 
	xmproc <- spawnPipe ("xmobar $HOME/.config/xmobar/xmobarrc.hs")
	xmonad $ docks defaults 


-- A structure containing your configuration settings, overriding
-- fields in the default config. Any you don't override, will
-- use the defaults defined in xmonad/XMonad/Config.hs
--
-- No need to modify this.
--
defaults = def {
      -- simple stuff
        terminal           = myTerminal,
        focusFollowsMouse  = myFocusFollowsMouse,
        clickJustFocuses   = myClickJustFocuses,
        borderWidth        = myBorderWidth,
        modMask            = myModMask,
        workspaces         = myWorkspaces,
        normalBorderColor  = myNormalBorderColor,
        focusedBorderColor = myFocusedBorderColor,

      -- key bindings
        keys               = myKeys,
        mouseBindings      = myMouseBindings,

      -- hooks, layouts
        layoutHook         = spacingRaw True (Border 0 5 5 5) True (Border 5 5 5 5) True $ myLayout,
        manageHook         = myManageHook,
        handleEventHook    = myEventHook,
        logHook            = myLogHook,
        startupHook        = myStartupHook
    }
